$ save_verify = 'f$verify(0)'   ! Change 0 to 1 for debugging.
$!
$!  TITLE       SLIP.COM
$!  IDENT       'V2.4, 10-22-1996'
$!
$!  Facility:   This product is one component of the SLIP_Extensions software
$!              distribution package for use with CMU-OpenVMS/IP.
$!              
$!  Abstract:   This DCL command procedure operates with a CKERMIT script to
$!              automate the process of establishing a temporary dialin SLIP
$!              connection to the Internet.  A discussion of the command-level
$!              changes occasioned by the calls between the command procedure
$!              and the script is included in the reference manual that is
$!              part of the SLIP_Extensions distribution package.
$!              
$!              This code *MUST* be modified to correctly restart any
$!              processes that die when the IPACP process is stopped and
$!              restarted.  Search the code for the phrases "!comment out"
$!              and "!uncomment" to find the lines that may need revision.
$!              Also review the "privileges" area near the top of the code.
$!              Finally, you *MUST* modify the Domain_Name and IP_Address
$!              definitions at the beginning of the code.
$!  
$!  Copyright  1995, 1996, Rob Puller.  All rights reserved.
$!
$!              Permission to use this product is granted subject to the terms
$!              and conditions set forth in the reference manual included with
$!              the software distribution package (which must be intact).
$!              
$!  Warnings:   No warranty of any type is provided.  No commitment is made,
$!              however, I'll fix reported bugs if time and resource permit.
$!              
$!              This software modifies (writes to) INTERNET.CONFIG and
$!              NAMRES.CONFIG, so make secure copies before using.
$!              It also facilitates connection to the Internet and it's
$!              well (sensationally) documented hazards.  Beware...
$!              
$!  Contact:    Rob Puller                      Robley S. Puller, P.E.
$!                                              Consulting Engineer
$!              rpuller@afn.org                 2130 N.W. 51st Terrace
$!              http://www.afn.org/~rpuller/    Gainesville, FL  32605
$!
$!  History:    Comments
$!  --------    --------------------------------------------------------------
$!  10-22-96    Write SLIP interface device address to NAMRES.CONFIG for Lynx.
$!  10-10-96    Added a timer to warn User of impending logout by ISP.
$!  10-05-96    Check privileges and call SLIP.INI from SLIP.COM.
$!  05-04-96    Automate acquisition of IP address.
$!  03-01-96    Add support for MGFTP.
$!  11-01-95    Initial release.
$!  
$!  Domain_Name and IP_Address definitions.  These *MUST* be modified.
$!  
$ Domain_Name = "rsp.eng.com"       ! Enter your Domain Name here.
$ Enet_Address = "192.168.100.100"  ! Enter "" if you have no ethernet device.
$ Slip_Address = "128.227.163."     ! Enter your Slip address prefix here.
$!  
$!  Provide some default actions and definitions.
$!  
$ on control then goto _exit
$ on error then goto _oper_err
$ say := write sys$output
$!
$!  Get the network interface device address if it's not on the command line.
$!
$ if P1 .eqs. ""
$ then
$   type sys$input

  An IP Address is a 32 bit number displayed as 4 octets in a dotted decimal
  notation.  Each octet represents the following values:

              +-------------- 1st octet - Network Address
              |   +---------- 2nd octet - Network Address
              |   |   +------ 3rd octet - Sub-net Address
              |   |   |   +-- 4th octet - Network Interface Device Address
              |   |   |   |
             nnn.nnn.sss.iii
             
  Enter [E] to Enable SLIP.
  Enter [D] to Disable SLIP.
  Enter your [Network Interface Device Address] if you are testing SLIP.

  You may enter these values on the command line in the future.

$   inquire P1 "  Enter your choice"
$ endif
$ P1 = f$edit(P1,"UPCASE")
$!
$!  Initiate actions to [E]nable SLIP if P1 equals "E".
$!
$ if P1 .eqs "E"
$ then
$!
$!  IPACP requires CMKRNL,DETACH,WORLD,SHARE,ACNT privileges.
$!  IPNCP requires PHY_IO,NETMBX privileges.
$!  MX Mailer requires CMKRNL,SYSPRV,SYSNAM privileges.
$!  MX SMTP_SERVER requires SYSLCK,SYSPRV,EXQUOTA,SYSNAM,PHY_IO privileges.
$!  System logical name table definitions require the SYSNAM privilege.
$!  
$!  Other applications may require additional privileges.  It is possible to
$!  specify "ALL" if it becomes too tedious to determine minimum privileges.
$!  
$!  Enable the required privileges if they are authorized for the User.  The
$!  User's existing privileges are re-established upon process completion.
$!
$ required_privileges = -
    "CMKRNL,DETACH,WORLD,SHARE,PHY_IO,NETMBX,SYSPRV,SYSNAM,SYSLCK,EXQUOTA,ACNT"
$ prev_privs = f$setprv(required_privileges)
$ if .not. f$privilege(required_privileges) then goto _no_privileges
$!
$!  Redirect input to allow SLIP.INI to acquire the password.  Returns to
$!  stdin upon completion of the next command.
$!  
$   define/user sys$input sys$command
$!
$!  Call SLIP.INI with CKERMIT to dial the Internet Service Provider and
$!  acquire a temporary serial port network interface device address.  Command
$!  and "required_privileges" will be inherited by SLIP.INI which will pass
$!  them to a new process when it spawns out of CKERMIT.
$!  
$   klip
$!
$!  Command will return here when the User disconnects the SLIP connection.
$!
$   goto _exit
$ endif
$!
$!  Initiate actions to [D]isable SLIP if P1 equals "D".
$!
$ if P1 .eqs. "D"
$ then
$   alarm_entry = f$getqui("DISPLAY_ENTRY","ENTRY_NUMBER","SLIP_ALARM","BATCH")
$   if alarm_entry .nes. "" then delete/entry='alarm_entry'
$   say ""
$   say "%IPNCP-S-IPACP_INFO, Disabling SLIP connections"
$   say ""
$   say ""
$   goto _begin
$ endif
$!
$!  Initiate actions to define the SLIP network interface to CMU_OpenVMS/IP
$!  if P1 equals an integer.
$!
$ if f$type(P1) .nes. "INTEGER" then goto _no_address
$ say ""
$ say "%IPNCP-S-IPACP_INFO, Enabling SLIP connections"
$ say ""
$ say ""
$!
$!  P1 must be an integer to get to this point, so that means that this is
$!  the 2nd generation of this procedure (called by SLIP.INI), that the ISP
$!  has started the SLIP interface on his end of the wire, and that he has
$!  started his timeout proceedure.  Get the terminal for this process and 
$!  pass it to our own timeout procedure which will warn us that our time
$!  is almost up so we can take action before the ISP logs us out.
$!  
$ if f$search("CMUIP_ROOT:[SYSMGR]SLIP_ALARM.COM") .nes. "" then -
    alarm_entry = f$getqui("DISPLAY_ENTRY","ENTRY_NUMBER","SLIP_ALARM","BATCH")
$ if alarm_entry .eqs. ""
$ then
$   top_pid = f$getjpi("","MASTER_PID")
$   term = f$getjpi(top_pid,"TERMINAL")
$   define/user/nolog sys$output nl:
$   submit/nolog/noprint/nokeep/parameter = 'term' -
        cmuip_root:[sysmgr]slip_alarm.com
$ endif
$!
$_begin:
$!
$!  At this point, P1 is either D or a three digit IP_address.  In either case,
$!  the IPACP should be stopped and restarted to allow the status of the serial
$!  port network interface device to be defined.  Processes that would die when
$!  IPACP is stopped should be stopped and restarted also.
$!  
$!  Shutdown the FTP Listener if it's running.
$!
$ ctx = ""
$ context = f$context( "PROCESS", ctx, "PRCNAM", "MGFTP Listener", "EQL" )
$ pid = f$pid( ctx )
$ if pid .nes. ""
$ then
$   stop/ident='pid'
$ endif
$!
$!  Shutdown the Name Resolver if it's running.
$!
$ ctx = ""
$ context = f$context( "PROCESS", ctx, "PRCNAM", "NAMRES", "EQL" )
$ pid = f$pid( ctx )
$ if pid .nes. ""
$ then
$   define/user/nolog sys$output nl:
$   ipncp namres exit
$ endif
$!
$!  Shutdown the IP Ancillary Control Process if it's up; this will also
$!  kill the SMTP Server.
$!
$ ctx = ""
$ context = f$context( "PROCESS", ctx, "PRCNAM", "IPACP(V6.6-5A)", "EQL" )
$ pid = f$pid( ctx )
$ if pid .nes. ""
$ then
$   define/user/nolog sys$output nl:
$   ipncp netexit
$ endif
$!
$!  Modify the Slip Driver Device definition in INTERNET.CONFIG.
$!
$ opened_file = "cmuip_root:[sysmgr]internet.config"
$ open /read /write /error = _open_err file cmuip_root:[sysmgr]internet.config
$_begin_loop_1:
$ read /end_of_file = _rec_not_found /error = _read_err /prompt="" file record
$ if f$locate(":CSDRV", record) .eq. f$length(record) then goto _begin_loop_1
$ if (f$locate(":CSDRV", record) .eq. 11) then record = ";''record'"
$ record = f$edit(record,"COLLAPSE")                    ! Delete the padding.
$ if P1 .eqs. "D" then goto _end_loop_1
$ part1 = f$element( 0, ":", record )
$ part1 = f$extract( 1, f$length( part1 ) - 1, part1 )  ! Uncomment the line.
$ part2 = f$element( 1, ":", record )
$ part3 = f$element( 2, ":", record )
$ part4 = f$element( 3, ":", record )
$ part5 = f$element( 4, ":", record )
$ part5a = f$element( 0, ".", part5 )
$ part5b = f$element( 1, ".", part5 )
$ part5c = f$element( 2, ".", part5 )
$ mask = f$element( 5, ":", record )
$ record = "''part1':''part2':''part3':''part4':''part5a'.''part5b'.''part5c'.''P1':''mask'"
$_end_loop_1:
$ ip_address = f$element( 4, ":", record )
$ if f$length( record ) .ne. 75
$ then
$   pad = f$extract( 0, 75 - f$length( record ), -
    "                                       " )
$   record = record + pad
$ endif
$ write /update /error = _write_err file record
$ close file
$!
$!  Modify the local host Domain_Server definition in NAMRES.CONFIG.
$!
$ opened_file = "cmuip_root:[sysmgr]namres.config"
$ open /read /write /error = _open_err file cmuip_root:[sysmgr]namres.config
$_begin_loop_2:
$ read /end_of_file = _rec_not_found /error = _read_err /prompt="" file record
$ if f$locate("com:''Slip_Address'", record) .eq. f$length(record) then goto _begin_loop_2
$ if (f$locate("com:''Slip_Address'", record) .eq. 24) then record = ";''record'"
$ record = f$edit(record,"COLLAPSE")                    ! Delete the padding.
$ if P1 .eqs. "D" then goto _end_loop_2
$ part1 = f$element( 0, ":", record )
$ part1 = f$extract( 1, f$length( part1 ) - 1, part1 )  ! Uncomment the line.
$ part2 = f$element( 1, ":", record )
$ part3 = f$element( 2, ":", record )
$ part4 = f$element( 3, ":", record )
$ part4a = f$element( 0, ".", part4 )
$ part4b = f$element( 1, ".", part4 )
$ part4c = f$element( 2, ".", part4 )
$ port = f$element( 4, ":", record )
$ record = "''part1':''part2':''part3':''part4a'.''part4b'.''part4c'.''P1':''port'"
$_end_loop_2:
$ if f$length( record ) .ne. 75
$ then
$   pad = f$extract( 0, 75 - f$length( record ), -
    "                                       " )
$   record = record + pad
$ endif
$ write /update /error = _write_err file record
$ close file
$!
$!  Define a Slip logical (SHO LOG SLIP tells user if Slip is enabled).
$!
$ if P1 .eqs. "D"
$ then
$   define/sys/nolog SLIP "Slip Connections disabled."
$   define/sys/exec/nolog INTERNET_HOST_NAME "''Domain_Name'","''Enet_Address'"
$ else
$   define/sys/nolog SLIP "Slip Connections enabled to ''ip_address'"
$   define/sys/exec/nolog INTERNET_HOST_NAME "''Domain_Name'",-
        "''Enet_Address'","''Slip_Address'''P1'"
$ endif
$!
$!  Make sure the ipacp process has exited before restarting.
$!
$_pause1:
$ ctx = ""
$ context = f$context( "PROCESS", ctx, "PRCNAM", "IPACP(V6.6-5A)", "EQL" )
$ pid = f$pid( ctx )
$ if pid .nes. ""
$ then
$   wait 00:00:01.00
$   goto _pause1
$ endif
$!
$!  Make sure the smtp_server process has exited before restarting.
$!
$_pause2:
$ ctx = ""
$ context = f$context( "PROCESS", ctx, "PRCNAM", "MX SMTP Server", "EQL" )
$ pid = f$pid( ctx )
$ if pid .nes. ""
$ then
$   wait 00:00:01.00
$   goto _pause2
$ endif
$!
$!  Determine if this is an initial startup or a restart.
$!
$ if F$TRNLNM("IPNCP$CONFIG") .eqs. ""
$ then
$   !
$   ! Its an initial startup.
$   !
$   say ""
$   @cmuip_root:[sysmgr]ip_startup.com
$   @madgoat_root:[com]ftp_start_listener   !comment out if you don't use MGFTP.
$   @sys$startup:mx_startup.com             !comment out if you don't use MX.
$   !
$   ! Start Domain during initial startup if you use it.  You do not need to
$   ! start and stop Domain on restarts because it will survive loss of IPACP.
$   !
$!  @$disk2:[domain]startup.com             !uncomment this if you use DOMAIN.
$ else
$   !
$   ! Its a restart.
$   !
$   say ""
$   run cmuip_root:[sysexe]ipacp.exe -
        /noaccountng -
        /noauthorize -
        /detached -
        /dump -
        /buffer_limit=65535 -
        /io_buffered=32767 -
        /io_direct=32767 -
        /priority=6 -
        /privileges=(noexquota,cmkrnl) -
        /file_limit=512 -
        /ast_limit=200 -
        /queue_limit=100 -
        /extent=2048 -
        /uic=[1,4] -
        /input=nl:/output=nl:/error=nl:
$   @madgoat_root:[com]ftp_start_listener   !comment out if you don't use MGFTP.
$   @sys$startup:mx_startup smtp_server     !comment out if you don't use MX.
$ endif
$!
$!  Wrap things up.
$!
$ say ""
$ if P1 .eqs. "D"
$ then
$   say "%IPNCP-S-IPACP_STATUS, IPACP running without SLIP"
$   say ""
$ else
$   say "%IPNCP-S-IPACP_STATUS, IPACP running with local SLIP address ''ip_address'"
$   say ""
$ endif
$!
$_exit:
$!
$!  Restore the User's original privileges and exit.
$!  
$ if "''prev_privs'" .nes. "" then prev_privs = f$setprv(prev_privs)
$ if P1 .eqs. "D" then LOGOUT
$ exit 1 .or. f$verify(save_verify)
$!
$!  Error handlers.
$!
$_oper_err:
$ say "  Error during command procedure processing, possibly caused by use"
$ say "  of Control_C to escape from Ckermit.  If this is not the case, try"
$ say "  setting the ""Verification On"" bit in the first line of SLIP.COM"
$ say "  to trace the operation and detect where the error occurs."
$ say ""
$ goto _exit
$!
$_no_privileges:
$ say "  Insufficient privileges to execute IPACP commands."
$ say "  Requires ",required_privileges," privileges."
$ say ""
$ goto _exit
$!
$_open_err:
$ say "  Error opening ''opened_file'."
$ say ""
$ goto _exit
$!
$_read_err:
$ close file
$ say "  Error reading ''opened_file'."
$ say ""
$ goto _exit
$!
$_write_err:
$ close file
$ say "  Error writing to ''opened_file'."
$ say ""
$ goto _exit
$!
$_rec_not_found:
$ close file
$ say "  Device driver specification not found in ''opened_file'."
$ say ""
$ goto _exit
$!
$_no_address:
$ say ""
$ say "  You must enter one of the following to invoke this procedure:"
$ say ""
$ say "  (E)nable  - Dials your ISP, acquires your IP address, and connects SLIP."
$ say "  (D)isable - Disconnects your SLIP connection."
$ say "   A three digit machine address (use only for testing purposes)."
$ say ""
$ exit
