$ save_verify = 'f$verify(0)'   ! Change 0 to 1 for debugging.
$!
$!  TITLE       SLIP_ALARM.COM
$!  IDENT       'V2.4, 10-10-1996'
$!
$!  Facility:   This product is one component of the SLIP_Extensions software
$!              distribution package for use with CMU-OpenVMS/IP.
$!              
$!  Abstract:   If this DCL command procedure is found, SLIP.COM will submit
$!              it to a batch queue where it will wait for a User-determined
$!              period of time.  Upon timeout, SLIP_ALARM issues a message.
$!              This "alarm clock" may be used to inform the User that his
$!              ISP is about to disconnect his SLIP line.  If an alarm is
$!              not needed, rename this file to something like SLIP.ALARM to
$!              prevent its use.
$!              
$!              This code *MUST* be modified to timeout when needed.  See the
$!              "modify" comment in the code below.  The "wait period" is
$!              usually set to 5 minutes less than the time allowed by the ISP.
$!              
$!  Copyright  1995, Rob Puller.  All rights reserved.
$!
$!              Permission to use this product is granted subject to the terms
$!              and conditions set forth in the reference manual included with
$!              the software distribution package (which must be intact).
$!              
$!  Warnings:   No warranty of any type is provided.  No commitment is made,
$!              however, I'll fix reported bugs if time and resource permit.
$!              
$!  Contact:    Rob Puller                      Robley S. Puller, P.E.
$!                                              Consulting Engineer
$!              rpuller@afn.org                 2130 N.W. 51st Terrace
$!              http://www.afn.org/~rpuller/    Gainesville, FL  32605
$!
$!  History:    Comments
$!  --------    --------------------------------------------------------------
$!  10-10-96    Initial release.
$!  
$!  Provide some default actions and definitions.
$!  
$ on control then goto _exit
$ on error then goto _oper_err
$!
$!  Reply/terminal requires OPER privilege.
$!
$ required_privileges = "OPER"
$ prev_privs = f$setprv(required_privileges)
$ if .not. f$privilege(required_privileges) then goto _no_privileges
$!
$!  Send a message that the ISP is about to shut down the SLIP connection.
$!
$ wait 00:55:00.00      ! <-- Modify.
$ bel[0,8] = 7
$ lf[0,8] = 10
$ reply/nonotify/terminal = 'P1' -
    "''lf'''bel'Your session will end in 5 minutes.''lf'"
$!
$!  Restore the User's original privileges and exit.
$!  
$ prev_privs = f$setprv(prev_privs)
$ exit 1 .or. f$verify(save_verify)
$!
$!  Error handler.
$!
$_no_privileges:
$ say "  Insufficient privilege to execute REPLY/TERMINAL command."
$ say "  Requires ",required_privileges," privileges."
$ say ""
$ goto _exit
